@@ -1,8 +1,8 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-app.c 21122 2006-04-20 19:58:31Z jasper $
+/*  $Id: panel-app.c 24093 2006-12-13 19:57:17Z jasper $
  *
- *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
+ *  Copyright © 2005-2006 Jasper Huijsmans <jasper@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published 
@@ -23,6 +23,10 @@
 #include <config.h>
 #endif
 
+#include <stdio.h> 
+#include <stdlib.h>
+#include <unistd.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <string.h>
 #include <errno.h>
@@ -31,6 +35,8 @@
 #include <X11/Xlib.h>
 #include <gtk/gtk.h>
 #include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4panel/xfce-panel-macros.h>
+#include <libxfce4panel/xfce-panel-convenience.h>
 
 #include "panel-app.h"
 #include "panel-app-messages.h"
@@ -50,11 +56,14 @@
 
 #define SELECTION_NAME "XFCE4_PANEL"
 #define PANEL_LAUNCHER "launcher"
+#define SAVE_TIMEOUT   30000
 
 #if defined(TIMER) && defined(G_HAVE_ISO_VARARGS)
 void
-xfce_panel_program_log (const char *file, const int line, 
-                        const char *format, ...)
+xfce_panel_program_log (const char *file, 
+                        const int   line, 
+                        const char *format, 
+                        ...)
 {
     va_list args;
     gchar  *formatted;
@@ -90,29 +99,28 @@ typedef struct _PanelApp PanelApp;
 
 struct _PanelApp
 {
-    GtkWidget *gtk_ipc_window;
-    Window ipc_window;
+    GtkWidget     *gtk_ipc_window;
+    Window         ipc_window;
 
     SessionClient *session_client;
-    PanelRunState runstate;
-    GPtrArray *panel_list;
-    GPtrArray *monitor_list;
+    PanelRunState  runstate;
+    GPtrArray     *panel_list;
+    GPtrArray     *monitor_list;
 
-    int check_id;
-    int save_id;
+    int            save_id;
+    int            current_panel;
+    GList         *dialogs;
 
-    int current_panel;
+    /* Initialization. Also unset before cleanup. */
+    guint          initialized:1; 
 
-    GList *dialogs;
-
-    guint initialized:1;
-
-    /* check whether monitors in Xinerama are aligned */
-    guint xinerama_and_equal_width:1;
-    guint xinerama_and_equal_height:1;
+    /* Check whether monitors in Xinerama are aligned. */
+    guint          xinerama_and_equal_width:1;
+    guint          xinerama_and_equal_height:1;
 };
 
 static PanelApp panel_app = {0};
+static int signal_pipe[2];
 
 
 /* cleanup */
@@ -120,9 +128,14 @@ static PanelApp panel_app = {0};
 static void
 cleanup_panels (void)
 {
-    int i;
+    int    i;
     GList *l;
 
+    if (!panel_app.initialized)
+        return;
+
+    panel_app.initialized = FALSE;
+
     l = panel_app.dialogs;
     panel_app.dialogs = NULL;
     
@@ -150,7 +163,7 @@ cleanup_panels (void)
     for (i = 0; i < panel_app.monitor_list->len; ++i)
     {
         XfceMonitor *xmon = g_ptr_array_index (panel_app.monitor_list, i);
-        g_free (xmon);
+        panel_slice_free (XfceMonitor, xmon);
     }
     g_ptr_array_free (panel_app.monitor_list, TRUE);
 }
@@ -184,35 +197,17 @@ sighandler (int sig)
      * that tests the flag.
      * This will prevent problems with gtk main loop threads and stuff
      */
-    switch (sig)
+    if (write (signal_pipe[1], &sig, sizeof (int)) != sizeof (int))
     {
-	case SIGUSR1:
-            DBG ("USR1 signal caught");
-	    panel_app.runstate = PANEL_RUN_STATE_RESTART;
-	    break;
-
-	case SIGUSR2:
-            DBG ("USR2 signal caught");
-	    panel_app.runstate = PANEL_RUN_STATE_QUIT;
-	    break;
-
-	case SIGINT:
-        case SIGABRT:
-            DBG ("INT or ABRT signal caught");
-            panel_app.runstate = PANEL_RUN_STATE_QUIT_NOSAVE;
-            break;
-
-	default:
-            DBG ("Signal caught: %d", sig);
-	    panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
+        g_printerr ("unix signal %d lost\n", sig);
     }
 }
 
 static gboolean
-check_signal_state (void)
+check_run_state (void)
 {
     static int recursive = 0;
-    gboolean quit = FALSE;
+    gboolean   quit      = FALSE;
  
     /* micro-optimization */
     if (G_LIKELY (panel_app.runstate == PANEL_RUN_STATE_NORMAL))
@@ -230,9 +225,11 @@ check_signal_state (void)
             panel_app_save ();
             quit = TRUE;
             break;
+
         case PANEL_RUN_STATE_QUIT_NOSAVE:
             quit = TRUE;
             break;
+
         default:
             if (panel_app.session_client)
             {
@@ -252,9 +249,12 @@ check_signal_state (void)
 
     if (quit)
     {
+        if (panel_app.save_id)
+            g_source_remove (panel_app.save_id);
+        panel_app.save_id = 0;
+
         /* this is necessary, because the function is not only called from the
          * timeout, so just returning FALSE is not enough. */
-        g_source_remove (panel_app.check_id);
         gtk_main_quit ();
         return FALSE;
     }
@@ -264,11 +264,84 @@ check_signal_state (void)
     return TRUE;
 }
 
+static gboolean
+set_run_state (GIOChannel   *source, 
+               GIOCondition  cond, 
+               gpointer      data)
+{
+    GError    *error = NULL;
+    GIOStatus  status;
+    gsize      bytes_read;
+    /* 
+     * There is no g_io_channel_read or g_io_channel_read_int, so we read
+     * char's and use a union to recover the unix signal number.
+     */
+    union
+    {
+        gchar chars[sizeof (int)];
+        int   signal;
+    } buf;
+
+    while ((status = g_io_channel_read_chars (source, buf.chars, sizeof (int),
+                                              &bytes_read, &error)
+           ) == G_IO_STATUS_NORMAL)
+    {
+        if (bytes_read != sizeof (int))
+        {
+            g_printerr ("lost data in signal pipe: expected %d, receieved %d",
+                        sizeof (int), bytes_read);
+            /* always at least quit if we received data */
+            panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
+            continue;
+        }
+
+        switch (buf.signal)
+        {
+            case SIGUSR1:
+                DBG ("USR1 signal caught");
+                panel_app.runstate = PANEL_RUN_STATE_RESTART;
+                break;
+
+            case SIGUSR2:
+                DBG ("USR2 signal caught");
+                panel_app.runstate = PANEL_RUN_STATE_QUIT;
+                break;
+
+            case SIGINT:
+            case SIGABRT:
+                DBG ("INT or ABRT signal caught");
+                panel_app.runstate = PANEL_RUN_STATE_QUIT_NOSAVE;
+                break;
+
+            default:
+                DBG ("Signal caught: %d", buf.signal);
+                panel_app.runstate = PANEL_RUN_STATE_QUIT_NOSAVE;
+        }
+    }
+
+    if (error != NULL)
+    {
+        g_printerr ("reading signal pipe failed: %s\n", error->message);
+        g_error_free (error);
+        panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
+    }
+    if (status != G_IO_STATUS_AGAIN)
+    {
+        g_printerr ("signal pipe has been closed\n");
+        panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
+    }
+
+    return check_run_state ();
+}
+
 /* session */
 
 static void
-session_save_yourself (gpointer data, int save_style, gboolean shutdown,
-                       int interact_style, gboolean fast)
+session_save_yourself (gpointer data, 
+                       int      save_style, 
+                       gboolean shutdown,
+                       int      interact_style, 
+                       gboolean fast)
 {
     panel_app_save ();
 }
@@ -280,12 +353,12 @@ session_die (gpointer client_data)
 }
 
 /* screen layout */
- static void
+static void
 monitor_size_changed (GdkScreen *screen)
 {
-    int i;
+    int          i;
     XfceMonitor *monitor;
-    GtkWidget *panel;
+    GtkWidget   *panel;
 
     for (i = 0; i < panel_app.monitor_list->len; ++i)
     {
@@ -310,11 +383,14 @@ monitor_size_changed (GdkScreen *screen)
 static void
 create_monitor_list (void)
 {
-    GdkDisplay *display;
-    GdkScreen *screen;
+    GdkDisplay  *display;
+    GdkScreen   *screen;
     XfceMonitor *monitor;
-    int n_screens, n_monitors = 0, i, j, w = 0, h = 0;
-    gboolean equal_w, equal_h;
+    int          n_screens;
+    int          n_monitors = 0;
+    int          i, j;
+    int          w = 0, h = 0;
+    gboolean     equal_w, equal_h;
     
     panel_app.monitor_list = g_ptr_array_new ();
 
@@ -332,7 +408,7 @@ create_monitor_list (void)
 
         for (j = 0; j < n_monitors; ++j)
         {
-            monitor = g_new0 (XfceMonitor, 1);
+            monitor = panel_slice_new0 (XfceMonitor);
 
             monitor->screen = screen;
             monitor->num = j;
@@ -353,7 +429,7 @@ create_monitor_list (void)
             h = monitor->geometry.height;
             
 #if TEST_MULTIPLE_MONITORS
-            monitor = g_new0 (XfceMonitor, 1);
+            monitor = panel_slice_new0 (XfceMonitor);
 
             monitor->screen = screen;
             monitor->num = j;
@@ -375,15 +451,14 @@ create_monitor_list (void)
     }
 
     /* check layout */
-    /* TODO: can this be optimized? does it need to be? */
     for (i = 0; i < panel_app.monitor_list->len; ++i)
     {
-        XfceMonitor *mon1 = g_ptr_array_index (panel_app.monitor_list, i);
-        
+        XfceMonitor *mon1, *mon2;
+
+        mon1 = g_ptr_array_index (panel_app.monitor_list, i);
+
         for (j = 0; j < panel_app.monitor_list->len; ++j)
         {
-            XfceMonitor *mon2;
-            
             if (j == i)
                 continue;
 
@@ -454,9 +529,10 @@ unregister_dialog (GtkWidget *dialog)
 int
 panel_app_init (void)
 {
-    Atom selection_atom, manager_atom;
-    GtkWidget *invisible;
-    XClientMessageEvent xev;
+    Atom                 selection_atom;
+    Atom                 manager_atom;
+    GtkWidget           *invisible;
+    XClientMessageEvent  xev;
 
     if (panel_app.initialized)
         return 0;
@@ -464,7 +540,6 @@ panel_app_init (void)
     panel_app.initialized = TRUE;
     
     selection_atom = XInternAtom (GDK_DISPLAY (), SELECTION_NAME, False);
-
     panel_app.ipc_window = XGetSelectionOwner (GDK_DISPLAY (), selection_atom);
 
     if (panel_app.ipc_window)
@@ -491,15 +566,15 @@ panel_app_init (void)
     
     manager_atom = XInternAtom (GDK_DISPLAY (), "MANAGER", False);
     
-    xev.type = ClientMessage;
-    xev.window = GDK_ROOT_WINDOW ();
+    xev.type         = ClientMessage;
+    xev.window       = GDK_ROOT_WINDOW ();
     xev.message_type = manager_atom;
-    xev.format = 32;
-    xev.data.l[0] = GDK_CURRENT_TIME;
-    xev.data.l[1] = selection_atom;
-    xev.data.l[2] = panel_app.ipc_window;
-    xev.data.l[3] = 0;	/* manager specific data */
-    xev.data.l[4] = 0;	/* manager specific data */
+    xev.format       = 32;
+    xev.data.l[0]    = GDK_CURRENT_TIME;
+    xev.data.l[1]    = selection_atom;
+    xev.data.l[2]    = panel_app.ipc_window;
+    xev.data.l[3]    = 0;	/* manager specific data */
+    xev.data.l[4]    = 0;	/* manager specific data */
 
     XSendEvent (GDK_DISPLAY (), GDK_ROOT_WINDOW (), False,
                 StructureNotifyMask, (XEvent *) & xev);
@@ -540,11 +615,61 @@ panel_app_init_panel (GtkWidget *panel)
  * Returns: 1 to restart and 0 to quit.
  **/
 int
-panel_app_run (int argc, char **argv)
+panel_app_run (int    argc, 
+               char **argv)
 {
 #ifdef HAVE_SIGACTION
-    struct sigaction act;
-    
+    struct sigaction  act;
+#endif    
+    GIOChannel       *g_signal_in;
+    GError           *error = NULL;
+    long              fd_flags;
+
+    /* create pipe and set writing end in non-blocking mode */
+    if (pipe (signal_pipe))
+    {
+        perror ("pipe");
+        return -1;
+    }
+    fd_flags = fcntl (signal_pipe[1], F_GETFL);
+    if (fd_flags == -1)
+    {
+        perror ("read descriptor flags");
+        return -1;
+    }
+    if (fcntl (signal_pipe[1], F_SETFL, fd_flags | O_NONBLOCK) == -1)
+    {
+        perror ("write descriptor flags");
+        return -1;
+    }
+
+    /* convert the reading end of the pipe into a GIOChannel */
+    g_signal_in = g_io_channel_unix_new (signal_pipe[0]);
+    g_io_channel_set_encoding (g_signal_in, NULL, &error);
+    if (error != NULL)
+    {                           /* handle potential errors */
+        g_printerr ("g_io_channel_set_encoding failed %s\n",
+                    error->message);
+        g_error_free (error);
+        return -1;
+    }
+
+    /* put the reading end also into non-blocking mode */
+    g_io_channel_set_flags (g_signal_in,
+                            g_io_channel_get_flags (g_signal_in) |
+                            G_IO_FLAG_NONBLOCK, &error);
+    if (error != NULL)
+    {
+        g_printerr ("g_io_set_flags failed %s\n", error->message);
+        g_error_free (error);
+        return -1;
+    }
+
+    /* register the reading end with the event loop */
+    g_io_add_watch (g_signal_in, G_IO_IN | G_IO_PRI, set_run_state, NULL);
+
+    /* register signals */
+#ifdef HAVE_SIGACTION
     act.sa_handler = sighandler;
     sigemptyset (&act.sa_mask);
 #ifdef SA_RESTART
@@ -552,19 +677,19 @@ panel_app_run (int argc, char **argv)
 #else
     act.sa_flags = 0;
 #endif
-    sigaction (SIGHUP, &act, NULL);
+    sigaction (SIGHUP,  &act, NULL);
     sigaction (SIGUSR1, &act, NULL);
     sigaction (SIGUSR2, &act, NULL);
-    sigaction (SIGINT, &act, NULL);
+    sigaction (SIGINT,  &act, NULL);
     sigaction (SIGABRT, &act, NULL);
     sigaction (SIGTERM, &act, NULL);
     act.sa_handler = sigchld_handler;
     sigaction (SIGCHLD, &act, NULL);
 #else
-    signal (SIGHUP, sighandler);
+    signal (SIGHUP,  sighandler);
     signal (SIGUSR1, sighandler);
     signal (SIGUSR2, sighandler);
-    signal (SIGINT, sighandler);
+    signal (SIGINT,  sighandler);
     signal (SIGABRT, sighandler);
     signal (SIGTERM, sighandler);
     signal (SIGCHLD, sigchld_handler);
@@ -580,7 +705,7 @@ panel_app_run (int argc, char **argv)
                             SESSION_RESTART_IF_RUNNING, 40);
 
     panel_app.session_client->save_yourself = session_save_yourself;
-    panel_app.session_client->die = session_die;
+    panel_app.session_client->die           = session_die;
 
     TIMER_ELAPSED("connect to session manager");
     if (!session_init (panel_app.session_client))
@@ -605,12 +730,11 @@ panel_app_run (int argc, char **argv)
                          NULL);
 
     /* Run Forrest, Run! */
-    panel_app.check_id = 
-        g_timeout_add (250, (GSourceFunc) check_signal_state, NULL);
     panel_app.runstate = PANEL_RUN_STATE_NORMAL;
     TIMER_ELAPSED("start main loop");
     gtk_main ();
     
+    /* cleanup */
     g_free (panel_app.session_client);
     panel_app.session_client = NULL;
 
@@ -644,58 +768,67 @@ save_timeout (void)
 void 
 panel_app_queue_save (void)
 {
+    if (!panel_app.initialized)
+        return;
+
     if (panel_app.runstate == PANEL_RUN_STATE_NORMAL)
     {
         if (!panel_app.save_id)
             panel_app.save_id = 
-                g_timeout_add (30000, (GSourceFunc)save_timeout, NULL);
+                g_timeout_add (SAVE_TIMEOUT, (GSourceFunc)save_timeout, NULL);
     }
 }
 
 void 
 panel_app_customize (void)
 {
-    panel_manager_dialog (panel_app.panel_list);
+    if (xfce_allow_panel_customization())
+        panel_manager_dialog (panel_app.panel_list);
 }
 
 void 
 panel_app_customize_items (GtkWidget *active_item)
 {
-    add_items_dialog (panel_app.panel_list, active_item);
+    if (xfce_allow_panel_customization())
+        add_items_dialog (panel_app.panel_list, active_item);
 }
 
 void 
 panel_app_save (void)
 {
-    panel_config_save_panels (panel_app.panel_list);
+    if (!panel_app.initialized)
+        return;
+
+    if (xfce_allow_panel_customization())
+        panel_config_save_panels (panel_app.panel_list);
 }
 
 void 
 panel_app_restart (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_RESTART;
-    check_signal_state ();
+    check_run_state ();
 }
 
 void 
 panel_app_quit (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_QUIT;
-    check_signal_state ();
+    check_run_state ();
 }
 
 void 
 panel_app_quit_noconfirm (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_QUIT_NOCONFIRM;
-    check_signal_state ();
+    check_run_state ();
 }
 
 void 
 panel_app_quit_nosave (void)
 {
     panel_app.runstate = PANEL_RUN_STATE_QUIT_NOSAVE;
-    check_signal_state ();
+    check_run_state ();
 }
 
 void 
@@ -703,6 +836,9 @@ panel_app_add_panel (void)
 {
     Panel *panel;
 
+    if (!xfce_allow_panel_customization())
+        return;
+
     panel = panel_new ();
 
     if (G_UNLIKELY (panel_app.panel_list == NULL))
@@ -725,9 +861,13 @@ panel_app_add_panel (void)
 void 
 panel_app_remove_panel (GtkWidget *panel)
 {
-    int response = GTK_RESPONSE_NONE, n;
+    int   response = GTK_RESPONSE_NONE;
+    int   n;
     char *first;
 
+    if (!xfce_allow_panel_customization())
+        return;
+        
     if (panel_app.panel_list->len == 1)
     {
         response = 
@@ -793,31 +933,32 @@ void
 panel_app_about (GtkWidget *panel)
 {
     XfceAboutInfo *info;
-    GtkWidget *dlg;
-    GdkPixbuf *pb;
+    GtkWidget     *dlg;
+    GdkPixbuf     *pb;
 
-    info = xfce_about_info_new (_("Xfce Panel"), "", _("Xfce Panel"), 
-                                XFCE_COPYRIGHT_TEXT ("2005", 
-                                                     "Jasper Huijsmans"),
-                                XFCE_LICENSE_GPL);
+    info = 
+        xfce_about_info_new (_("Xfce Panel"), "", _("Xfce Panel"), 
+                             XFCE_COPYRIGHT_TEXT ("2006", "Jasper Huijsmans"),
+                             XFCE_LICENSE_GPL);
 
     xfce_about_info_set_homepage (info, "http://www.xfce.org");
 
     xfce_about_info_add_credit (info, "Jasper Huijsmans", "jasper@xfce.org",
                                 _("Developer"));
 
+    xfce_about_info_add_credit (info, "Nick Schermer", "nick@xfce.org",
+                                _("Developer"));
+
     pb = xfce_themed_icon_load ("xfce4-panel", 48);
     dlg = xfce_about_dialog_new_with_values (NULL, info, pb);
-    gtk_window_set_screen (GTK_WINDOW (dlg),
-                           gtk_widget_get_screen (panel));
-    g_object_unref (pb);
+    g_object_unref (G_OBJECT (pb));
 
+    gtk_window_set_screen (GTK_WINDOW (dlg), gtk_widget_get_screen (panel));
     gtk_widget_set_size_request (dlg, 400, 300);
 
     gtk_dialog_run (GTK_DIALOG (dlg));
 
     gtk_widget_destroy (dlg);
-
     xfce_about_info_free (info);
 }
 
