@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel-properties.c 25300 2007-03-27 12:06:38Z nick $
+/*  $Id: panel-properties.c 25940 2007-07-30 19:22:58Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -242,6 +242,14 @@ _set_borders (Panel *panel)
                                        top, bottom, left, right);
 }
 
+/* fairly arbitrary maximum strut size */
+static gboolean
+_validate_strut( gint width,
+                 gint offset )
+{
+    return ( offset < width / 4 );
+}
+
 static void
 _set_struts (Panel       *panel, 
              XfceMonitor *xmon, 
@@ -279,12 +287,13 @@ _set_struts (Panel       *panel,
             /* no struts possible on Xinerama screens when this monitor
              * has neighbors on the left (see fd.o spec).
              */
-            if (!xmon->has_neighbor_left)
+            if (!xmon->has_neighbor_left 
+                && _validate_strut (xmon->geometry.width, w))
             {
                 data[0] = xmon->geometry.x 
                           + w;          /* left           */
                 data[4] = y;            /* left_start_y   */
-                data[5] = y + h;        /* left_end_y     */
+                data[5] = y + h - 1;    /* left_end_y     */
             }
         }
         else if (xfce_screen_position_is_right (priv->screen_position))
@@ -292,13 +301,14 @@ _set_struts (Panel       *panel,
             /* no struts possible on Xinerama screens when this monitor
              * has neighbors on the right (see fd.o spec).
              */
-            if (!xmon->has_neighbor_right)
+            if (!xmon->has_neighbor_right 
+                && _validate_strut (xmon->geometry.width, w))
             {
                 data[1] =  gdk_screen_get_width (xmon->screen) 
                            - xmon->geometry.x - xmon->geometry.width 
                            + w;         /* right          */
                 data[6] = y;            /* right_start_y  */
-                data[7] = y + h;        /* right_end_y    */
+                data[7] = y + h - 1;    /* right_end_y    */
             }
         }
         else if (xfce_screen_position_is_top (priv->screen_position))
@@ -306,12 +316,13 @@ _set_struts (Panel       *panel,
             /* no struts possible on Xinerama screens when this monitor
              * has neighbors on the top (see fd.o spec).
              */
-            if (!xmon->has_neighbor_above)
+            if (!xmon->has_neighbor_above 
+                && _validate_strut (xmon->geometry.height, h))
             {
                 data[2] = xmon->geometry.y 
                           + h;          /* top            */
                 data[8] = x;            /* top_start_x    */
-                data[9] = x + w;        /* top_end_x      */
+                data[9] = x + w - 1;    /* top_end_x      */
             }
         }
         else
@@ -319,13 +330,14 @@ _set_struts (Panel       *panel,
             /* no struts possible on Xinerama screens when this monitor
              * has neighbors on the bottom (see fd.o spec).
              */
-            if (!xmon->has_neighbor_below)
+            if (!xmon->has_neighbor_below
+                && _validate_strut (xmon->geometry.height, h))
             {
                 data[3] = gdk_screen_get_height (xmon->screen) 
                            - xmon->geometry.y - xmon->geometry.height 
                            + h;         /* bottom         */
                 data[10] = x;           /* bottom_start_x */
-                data[11] = x + w;       /* bottom_end_x   */
+                data[11] = x + w - 1;   /* bottom_end_x   */
             }
         }
     }
@@ -338,15 +350,6 @@ _set_struts (Panel       *panel,
          data[4], data[5], data[6], data[7], 
          data[8], data[9], data[10], data[11]);
 
-    /* Check for invalid values. */
-    for (i = 0; i < 4; i++)
-    {
-        if (data[i] > MIN(xmon->geometry.width,xmon->geometry.height) / 4)
-        {
-            return;
-        }
-    }
-    
     /* Check if values have changed. */
     for (i = 0; i < 12; i++)
     {
@@ -360,17 +363,17 @@ _set_struts (Panel       *panel,
     if (update)
     {
         gdk_error_trap_push ();
-
         gdk_property_change (GTK_WIDGET (panel)->window,
                              gdk_atom_intern ("_NET_WM_STRUT_PARTIAL", FALSE),
                              gdk_atom_intern ("CARDINAL", FALSE), 32,
                              GDK_PROP_MODE_REPLACE, (guchar *) & data, 12);
+        gdk_error_trap_pop ();
 
+        gdk_error_trap_push ();
         gdk_property_change (GTK_WIDGET (panel)->window,
                              gdk_atom_intern ("_NET_WM_STRUT", FALSE),
                              gdk_atom_intern ("CARDINAL", FALSE), 32,
                              GDK_PROP_MODE_REPLACE, (guchar *) & data, 4);
-
         gdk_error_trap_pop ();
     }
     
@@ -966,6 +969,7 @@ panel_init_position (Panel *panel)
     
     if (priv->full_width > XFCE_PANEL_NORMAL_WIDTH)
     {
+        xfce_itembar_set_allow_expand (XFCE_ITEMBAR (priv->itembar), TRUE);
         if (xfce_screen_position_is_horizontal (priv->screen_position))
         {
             int w;
@@ -1196,10 +1200,14 @@ panel_set_full_width (Panel *panel,
             {
                 case XFCE_PANEL_NORMAL_WIDTH:
                     w = h = -1;
+                    xfce_itembar_set_allow_expand (XFCE_ITEMBAR (priv->itembar),
+                                                   FALSE);
                     break;
                     
                 case XFCE_PANEL_FULL_WIDTH:
                 case XFCE_PANEL_SPAN_MONITORS:
+                    xfce_itembar_set_allow_expand (XFCE_ITEMBAR (priv->itembar), 
+                                                   TRUE);
                     if (xfce_screen_position_is_horizontal (
                                 priv->screen_position))
                     {
