@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: panel.h 20070 2006-02-25 22:33:57Z jasper $
+/*  $Id: panel.h 24092 2006-12-13 19:07:59Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -40,7 +40,6 @@ G_BEGIN_DECLS
 
 typedef struct _Panel               Panel;
 typedef struct _PanelClass          PanelClass;
-typedef struct _XfcePanelItemConfig XfcePanelItemConfig;
 
 struct _Panel
 {
@@ -54,12 +53,6 @@ struct _PanelClass
     XfcePanelWindowClass parent_class;
 };
 
-struct _XfcePanelItemConfig
-{
-    const char *name;
-    const char *id;
-};
-
 GType panel_get_type (void) G_GNUC_CONST;
 
 Panel *panel_new (void);
@@ -76,7 +69,7 @@ GtkWidget *panel_add_item_with_id (Panel *panel, const char *name,
                                    const char *id);
 
 /* configuration */
-XfcePanelItemConfig *panel_get_item_config_list (Panel *panel);
+GList *panel_get_item_list (Panel *panel);
 
 void panel_save_items (Panel *panel);
 
